/*
  garon -- A simple tool for formatting USB sticks.
  
  Copyright (C) 2007  Claudio Saavedra <csaavedra@alumos.utalca.cl>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <gtk/gtk.h>
#include <dbus/dbus.h>
#include <libhal.h>
#include <glib/gi18n.h>

#include "garon-combobox-devices.h"

enum {
	GARON_COL_PIXBUF,
	GARON_COL_DEVICE,
	GARON_COL_INFO,
	GARON_COL_UDI,
	GARON_COL_NUM
};

#define HAL_DRIVE_KEY      "storage.drive_type"
#define HAL_DRIVE_VALUE    "disk"
#define HAL_REMOVABLE_PROP "storage.removable"
#define HAL_DEVICE_KEY     "block.device"
#define HAL_PRODUCT_KEY    "info.product"

static void garon_combobox_devices_destroy (gpointer data);

static void
on_device_added (LibHalContext *hal_ctx,
		 const char *udi)
{
	g_print ("device added\n");
}

static void
on_device_removed (LibHalContext *hal_ctx,
		   const char *udi)
{
	g_print ("device removed\n");
}

void
garon_combobox_init (GtkComboBox *combo)
{
	DBusError error;
	DBusConnection *conn;
	LibHalContext *hal_ctx;
	
	dbus_error_init (&error);
	if (!(conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error))) {
		g_warning ("dbus error: %s: %s\n", error.name, error.message);
		LIBHAL_FREE_DBUS_ERROR (&error);
		return;
	}

	if (!(hal_ctx = libhal_ctx_new ())) {
		g_warning (_("Could not create HAL context.\n"));
		return;
	}

	if (!libhal_ctx_set_dbus_connection (hal_ctx, conn)) {
		g_warning (_("Could not set DBus connection.\n"));
		return;
	}
	
	if (!libhal_ctx_init(hal_ctx, &error)) {
		if (dbus_error_is_set(&error)) {
			g_warning (_("Error: libhal_ctx_init: %s: %s\n"), error.name, error.message);
			dbus_error_free (&error);
		}
		g_warning (_("Could not initialise connection to hald.\n"
			     "Normally this means the HAL daemon (hald) is not running or not ready.\n"));
		return;
	}

	libhal_ctx_set_user_data (hal_ctx, combo);
	
	libhal_ctx_set_device_added (hal_ctx, on_device_added);
	libhal_ctx_set_device_removed (hal_ctx, on_device_removed);

	g_object_set_data_full (G_OBJECT (combo), "hal-ctx", hal_ctx,
				garon_combobox_devices_destroy);
	
	dbus_error_free(&error);
}

void
garon_combobox_devices_prepare (GtkComboBox *combo)
{
	GtkListStore *store;
	GtkCellRenderer *renderer;

	store = gtk_list_store_new (GARON_COL_NUM,
				    GDK_TYPE_PIXBUF,
				    G_TYPE_STRING,
				    G_TYPE_STRING,
				    G_TYPE_STRING);
	
	gtk_combo_box_set_model (GTK_COMBO_BOX (combo),
				 GTK_TREE_MODEL (store));

	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
				    renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo),
					renderer,
					"pixbuf", GARON_COL_PIXBUF,
					NULL);
	
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
				    renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo),
					renderer,
					"text", GARON_COL_INFO,
					NULL);
}

void
garon_combobox_devices_populate (GtkComboBox *combo)
{
	LibHalContext *hal_ctx;
	gchar **device_list;
	gint num_devices;
	gint i;
	GtkListStore *store;
	GtkTreeIter iter;
	GtkWidget *cellview;
	GdkPixbuf *pixbuf;
	gchar *device, *info;

	hal_ctx = (LibHalContext *) g_object_get_data (G_OBJECT (combo), "hal-ctx");
	
	device_list = libhal_manager_find_device_string_match (hal_ctx,
							       HAL_DRIVE_KEY,
							       HAL_DRIVE_VALUE,
							       &num_devices, 
							       NULL); 

	store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)));

	cellview = gtk_cell_view_new ();
	pixbuf = gtk_widget_render_icon (cellview, GTK_STOCK_HARDDISK,
					 GTK_ICON_SIZE_BUTTON, NULL);
		
	for (i = 0; i < num_devices; i++) {
		if (libhal_device_get_property_bool (hal_ctx,
						     device_list[i],
						     HAL_REMOVABLE_PROP,
						     NULL)) {

			device = libhal_device_get_property_string (hal_ctx,
								    device_list[i], 
								    HAL_DEVICE_KEY,
								    NULL);
			info = libhal_device_get_property_string (hal_ctx,
								  device_list[i], 
								  HAL_PRODUCT_KEY,
								  NULL);
			
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter,
					    GARON_COL_PIXBUF, pixbuf,
					    GARON_COL_DEVICE, device,
					    GARON_COL_INFO, info,
					    GARON_COL_UDI, device_list[i],
					    -1);
			libhal_free_string (device);
			libhal_free_string (info);
		}
	}
	libhal_free_string_array (device_list);
	g_object_unref (pixbuf);
}

gchar *
garon_combobox_devices_get_selected (GtkComboBox *combo)
{
	GtkTreeModel *store;
	GtkTreeIter iter;
	gchar *device = NULL;

	store = gtk_combo_box_get_model (combo);
	if (gtk_combo_box_get_active_iter (combo, &iter)) {
		gtk_tree_model_get (store, &iter,
				    GARON_COL_DEVICE, &device,
				    -1);
	}

	return device;
}

gboolean
garon_combobox_devices_lock_selected (GtkComboBox *combo)
{
	GtkTreeModel *store;
	GtkTreeIter iter;
	gchar *udi = NULL;
	gchar *reason;
	gboolean done;
	LibHalContext *hal_ctx;
	
	hal_ctx = (LibHalContext *) g_object_get_data (G_OBJECT (combo), "hal-ctx");
	
	store = gtk_combo_box_get_model (combo);
	if (gtk_combo_box_get_active_iter (combo, &iter)) {
		gtk_tree_model_get (store, &iter,
				    GARON_COL_UDI, &udi,
				    -1);
	} else {
		return FALSE;
	}

	if (udi) {
		done = libhal_device_lock (hal_ctx, udi,
					   _("Device is being formatted."),
					   &reason, NULL);
		g_free (udi);
	}

	if (!done) {
		g_warning ("%s", reason);
		g_free (reason);
	}
	
	return done;
}

void
garon_combobox_devices_unlock_selected (GtkComboBox *combo)
{
	GtkTreeModel *store;
	GtkTreeIter iter;
	gchar *udi = NULL;
	LibHalContext *hal_ctx;
	
	hal_ctx = (LibHalContext *) g_object_get_data (G_OBJECT (combo), "hal-ctx");
       	store = gtk_combo_box_get_model (combo);
	if (gtk_combo_box_get_active_iter (combo, &iter)) {
		gtk_tree_model_get (store, &iter,
				    GARON_COL_UDI, &udi,
				    -1);
	}

	if (udi) {
		libhal_device_unlock (hal_ctx, udi, NULL);
		g_free (udi);
	}
}

static void
garon_combobox_devices_destroy (gpointer data)
{
	DBusError error;
	DBusConnection *conn;
	LibHalContext *hal_ctx;

	dbus_error_init (&error);
	hal_ctx = (LibHalContext *) data;

	conn = libhal_ctx_get_dbus_connection (hal_ctx);
	libhal_ctx_shutdown (hal_ctx, &error);
	libhal_ctx_free (hal_ctx);
	dbus_connection_unref (conn);
	dbus_error_free (&error);
}
