/*
  garon -- A simple tool for formatting USB sticks.
  
  Copyright (C) 2007  Claudio Saavedra <csaavedra@alumos.utalca.cl>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <glib/gi18n.h>
#include <glib/gprintf.h>
#include <string.h>

#include <config.h>

#include "garon.h"
#include "garon-combobox-devices.h"

#define GARON_GLADE_FILE GARON_DATA_DIR G_DIR_SEPARATOR_S "garon.glade"

#define GARON_WINDOW_GET_PRIVATE(object)				\
	G_TYPE_INSTANCE_GET_PRIVATE (object, GARON_TYPE_WINDOW, GaronWindowPrivate);

G_DEFINE_TYPE (GaronWindow, garon_window, GTK_TYPE_DIALOG)

struct _GaronWindowPrivate {
	GtkWidget *combo_device;
	GtkWidget *combo_fs;
	GtkWidget *entry_label;
	GtkWidget *checkbutton_quickformat;
};

typedef enum {
	GARON_FS_EXT3 = 0,
	GARON_FS_FAT32,
	GARON_FS_SUPPORTED
} GaronFileSystemId;

typedef struct {
	gchar *name;          /* A translatable name. */
	GaronFileSystemId id; /* A short code.        */
	gchar *code;          /* The filesystem ID.   */
	gchar *cmd;           /* Command for formatting. */
	gchar *par_label;     /* Parameter to specify the label. */
	gchar *par_extra;     /* Extra parameters needed */
} GaronFileSystem;

/* FIXME: Max 11 labels for vfat filesystem.
          Max 16 labels for ext3 filesystem.
*/
static const GaronFileSystem garon_supported_fs [GARON_FS_SUPPORTED] = {
	{ N_("Linux Ext3"),    GARON_FS_EXT3,  "83", "mkfs.ext3", "-L", "-v"},
	{ N_("Windows FAT32"), GARON_FS_FAT32, "0b",  "mkdosfs", "-n", "-v -F 32"}
};

#define GARON_SFDISK_CMD "sfdisk"

static void
garon_combobox_fs_prepare (GtkComboBox *combo)
{
	GtkListStore *store;
	GtkCellRenderer *renderer;
	GtkTreeIter iter;
	int i;
	
	store = gtk_list_store_new (2,
				    G_TYPE_STRING,
				    G_TYPE_POINTER);
	
	gtk_combo_box_set_model (GTK_COMBO_BOX (combo),
				 GTK_TREE_MODEL (store));

	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
				    renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo),
					renderer,
					"text", 0,
					NULL);

	for (i = 0; i < GARON_FS_SUPPORTED; i++) {
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter,
				    0, _(garon_supported_fs [i].name),
				    1, &garon_supported_fs[i],
				    -1);
	}
}

static void
garon_window_init (GaronWindow *window)
{
	GaronWindowPrivate *priv;
	GladeXML *xml;
	GtkWidget *vbox1;

	priv = window->priv = GARON_WINDOW_GET_PRIVATE (window);
	
	xml = glade_xml_new (GARON_GLADE_FILE, "vbox1", NULL);

	priv->combo_device = glade_xml_get_widget (xml, "combobox_device");
	priv->combo_fs = glade_xml_get_widget (xml, "combobox_fs");
	priv->entry_label = glade_xml_get_widget (xml, "entry_label");
	priv->checkbutton_quickformat = glade_xml_get_widget (xml, "checkbutton_quickformat");

	garon_combobox_init (GTK_COMBO_BOX (priv->combo_device));
	garon_combobox_devices_prepare (GTK_COMBO_BOX (priv->combo_device));
	garon_combobox_devices_populate (GTK_COMBO_BOX (priv->combo_device));

	garon_combobox_fs_prepare (GTK_COMBO_BOX (priv->combo_fs));

	vbox1 = glade_xml_get_widget (xml, "vbox1");

	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(window)->vbox),
			   vbox1);

}

static void
garon_window_class_init (GaronWindowClass *class)
{
	GObjectClass *g_object_class = G_OBJECT_CLASS (class);
	g_type_class_add_private (g_object_class, sizeof (GaronWindowPrivate));
}

GtkWidget *garon_window_new ()
{
	GaronWindow *window;

	window = GARON_WINDOW (g_object_new (GARON_TYPE_WINDOW,
					     "resizable", FALSE,
					     NULL));
	
	gtk_dialog_add_buttons (GTK_DIALOG (window),
				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
				_("Format"), GTK_RESPONSE_OK,
				NULL);

	return GTK_WIDGET (window);
}

static void
garon_window_format (GaronWindow *garon)
{
	gboolean quick_format;
	gchar *label;
	gchar *device;
	GaronWindowPrivate *priv;
	GaronFileSystem *fs;
	GtkTreeIter iter;
	GtkTreeModel *store;
	gchar command [256];
	gchar *std_out;
	gchar *std_err;
	gchar *cmd;
	gint e_status;
	GError *error = NULL;
			
	priv = garon->priv;
	
	quick_format = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->checkbutton_quickformat));
	label = g_strdup (gtk_entry_get_text (GTK_ENTRY (priv->entry_label)));

	garon_combobox_devices_lock_selected (GTK_COMBO_BOX (priv->combo_device));
	device = garon_combobox_devices_get_selected (GTK_COMBO_BOX (priv->combo_device));

	if (device == NULL) {
		g_warning (_("No device selected. Quiting..."));
		garon_combobox_devices_unlock_selected (GTK_COMBO_BOX (priv->combo_device));
		g_free (label);
		return;
	}
	
	g_print ("device: %s\n"
		 "label:  %s\n"
		 "qf:     %i\n",
		 device, label, quick_format);

	store = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->combo_fs));
	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->combo_fs), &iter)) {
		g_warning (_("No device selected. Quiting..."));
		garon_combobox_devices_unlock_selected (GTK_COMBO_BOX (priv->combo_device));
		g_free (device);
		g_free (label);
		return;
	}
	
	gtk_tree_model_get (store, &iter,
			    1, &fs,
			    -1);

	g_print ("the selected filesystem is %s\n", fs->name);

	if (!quick_format) {
		cmd = g_find_program_in_path (GARON_SFDISK_CMD);
		
		g_sprintf (command, "sh -c 'echo \",,%s\\n\" | %s %s'",
			   fs->code, cmd, device);

		g_free (cmd);

 		g_printf ("%s\n", command);
		if (!g_spawn_command_line_sync (command, &std_out, &std_err, &e_status, &error)) {
			g_warning ("%s", error->message);
			g_error_free (error);
		}
		g_print ("stdout:\n%s", std_out);
		g_print ("stderr:\n%s", std_err);
		g_free (std_out);
		g_free (std_err);
	}

	cmd = g_find_program_in_path (fs->cmd);

	if (fs->par_extra && strlen (label) > 0) {
		g_sprintf (command, "%s %s %s %s %s1",
			   cmd, fs->par_extra, fs->par_label, label, device);
	} else if (fs->par_extra) {
		g_sprintf (command, "%s %s %s1",
			   cmd, fs->par_extra, device);
	} else if (strlen (label) > 0) {
		g_sprintf (command, "%s %s %s %s1",
			   cmd, fs->par_label, label, device);
	} else {
		g_sprintf (command, "%s %s1",
			   cmd, device);
	}
	
	g_free (cmd);
	g_free (label);
	g_free (device);

	g_print ("about to execute: %s\n", command);

	if (!g_spawn_command_line_sync (command, &std_out, &std_err, &e_status, &error)) {
		g_warning ("%s", error->message);
		g_error_free (error);
	}

	g_print ("stdout:\n%s", std_out);
	g_print ("stderr:\n%s", std_err);
	g_free (std_out);
	g_free (std_err);
	
	garon_combobox_devices_unlock_selected (GTK_COMBO_BOX (priv->combo_device));
}

static gboolean
check_programs_availability (void)
{
	int i;
	gchar *cmd;
	gboolean ok = TRUE;
	
	for (i = 0; i < GARON_FS_SUPPORTED; i++) {
		cmd = g_find_program_in_path (garon_supported_fs[i].cmd);
		if (cmd == NULL) {
			g_warning (_("Missing required program: %s. "
				     "Formatting to %s filesystem is not available."),
				   garon_supported_fs[i].cmd, _(garon_supported_fs[i].name));
			ok = FALSE;
		} else {
			g_free (cmd);
		}
	}
	
	cmd = g_find_program_in_path (GARON_SFDISK_CMD);
	if (cmd == NULL) {
		g_warning (_("Missing required program: %s. "
			     "Partition table construction is not available."),
			   GARON_SFDISK_CMD);
		ok = FALSE;
	} else {
		g_free (cmd);
	}
	
	return ok;
}

int
main (gint argc, gchar **argv)
{
	GtkWidget *garon;
	gint result;
	
	gtk_init (&argc, &argv);
	bindtextdomain (PACKAGE, GARON_LOCALE_DIR);
	bind_textdomain_codeset (PACKAGE, "UTF-8");
	textdomain (PACKAGE);

	if (!check_programs_availability ()) {
		return 1;
	}

	garon = garon_window_new ();
	gtk_container_set_border_width (GTK_CONTAINER (garon),
					5);
	gtk_window_set_title (GTK_WINDOW (garon),
			      _("Format USB stick"));
	
	result = gtk_dialog_run (GTK_DIALOG (garon));
	switch (result) {
	case GTK_RESPONSE_OK:
		garon_window_format (GARON_WINDOW (garon));
		break;
	}
	
	gtk_widget_destroy (GTK_WIDGET (garon));
	return 0;
}
